@@ -15,6 +15,7 @@ const (
 	notRunning = iota
 	normal
 	paused
+	terminated
 )
 
 type supervisorID uint32
@@ -57,6 +58,16 @@ type ServiceToken struct {
 	id uint64
 }
 
+type UnstoppedService struct {
+	Service      Service
+	Name         string
+	ServiceToken ServiceToken
+}
+
+// An UnstoppedServiceReport will be returned by StopWithReport, reporting
+// which services failed to stop.
+type UnstoppedServiceReport []UnstoppedService
+
 type serviceWithName struct {
 	Service Service
 	name    string
@@ -138,6 +149,7 @@ type Supervisor struct {
 	serviceCounter       serviceID
 	control              chan supervisorMessage
 	liveness             chan struct{}
+	notifyServiceDone    chan serviceID
 	resumeTimer          <-chan time.Time
 
 	LogBadStop BadStopLogger
@@ -268,6 +280,7 @@ func New(name string, spec Spec) (s *Supervisor) {
 
 	s.control = make(chan supervisorMessage)
 	s.liveness = make(chan struct{})
+	s.notifyServiceDone = make(chan serviceID)
 	s.services = make(map[serviceID]serviceWithName)
 	s.servicesShuttingDown = make(map[serviceID]serviceWithName)
 	s.restartQueue = make([]serviceID, 0, 1)
@@ -398,8 +411,8 @@ func (s *Supervisor) Add(service Service) ServiceToken {
 	return ServiceToken{uint64(s.id)<<32 | uint64(<-response)}
 }
 
-// ServeBackground starts running a supervisor in its own goroutine. This
-// method does not return until it is safe to use .Add() on the Supervisor.
+// ServeBackground starts running a supervisor in its own goroutine. When
+// this method returns, the supervisor is guaranteed to be in a running state.
 func (s *Supervisor) ServeBackground() {
 	go s.Serve()
 	s.sync()
@@ -417,21 +430,27 @@ func (s *Supervisor) Serve() {
 		panic("Can't call Serve on an incorrectly-constructed *suture.Supervisor")
 	}
 
+	s.Lock()
+	if s.state == terminated {
+		// Got stopped before we got started.
+		s.Unlock()
+		return
+	}
+
+	if s.state != notRunning {
+		s.Unlock()
+		panic("Called .Serve() on a supervisor that is already Serve()ing")
+	}
+
+	s.state = normal
+	s.Unlock()
+
 	defer func() {
 		s.Lock()
 		s.state = notRunning
 		s.Unlock()
 	}()
 
-	s.Lock()
-	if s.state != notRunning {
-		s.Unlock()
-		panic("Running a supervisor while it is already running?")
-	}
-
-	s.state = normal
-	s.Unlock()
-
 	// for all the services I currently know about, start them
 	for _, id := range s.restartQueue {
 		namedService, present := s.services[id]
@@ -468,12 +487,9 @@ func (s *Supervisor) Serve() {
 
 				msg.response <- id
 			case removeService:
-				s.removeService(msg.id, msg.notification, s.control)
-			case serviceTerminated:
-				delete(s.servicesShuttingDown, msg.id)
+				s.removeService(msg.id, msg.notification)
 			case stopSupervisor:
-				s.stopSupervisor()
-				msg.done <- struct{}{}
+				msg.done <- s.stopSupervisor()
 				return
 			case listServices:
 				services := []Service{}
@@ -488,6 +504,8 @@ func (s *Supervisor) Serve() {
 				// used only by tests
 				panic("Panicking as requested!")
 			}
+		case serviceEnded := <-s.notifyServiceDone:
+			delete(s.servicesShuttingDown, serviceEnded)
 		case <-s.resumeTimer:
 			// We're resuming normal operation after a pause due to
 			// excessive thrashing
@@ -509,15 +527,56 @@ func (s *Supervisor) Serve() {
 	}
 }
 
+// StopWithReport will stop the supervisor like calling Stop, but will also
+// return a struct reporting what services failed to stop. This fully
+// encompasses calling Stop, so do not call Stop and StopWithReport any
+// more than you should call Stop twice.
+//
+// WARNING: Technically, any use of the returned data structure is a
+// TOCTOU violation:
+// https://en.wikipedia.org/wiki/Time-of-check_to_time-of-use
+// Since the data structure was generated and returned to you, any of these
+// services may have stopped since then.
+//
+// However, this can still be useful information at program teardown
+// time. For instance, logging that a service failed to stop as expected is
+// still useful, as even if it shuts down later, it was still later than
+// you expected.
+//
+// But if you cast the Service objects back to their underlying objects and
+// start trying to manipulate them ("shut down harder!"), be sure to
+// account for the possibility they are in fact shut down before you get
+// them.
+//
+// If there are no services to report, the UnstoppedServiceReport will be
+// nil. A zero-length constructed slice is never returned.
+//
+// Calling this on an already-stopped supervisor is invalid, but will
+// safely return nil anyhow.
+func (s *Supervisor) StopWithReport() UnstoppedServiceReport {
+	s.Lock()
+	if s.state == notRunning {
+		s.state = terminated
+		s.Unlock()
+		return nil
+	}
+	s.state = terminated
+	s.Unlock()
+
+	done := make(chan UnstoppedServiceReport)
+	if s.sendControl(stopSupervisor{done}) {
+		return <-done
+	}
+	return nil
+}
+
 // Stop stops the Supervisor.
 //
 // This function will not return until either all Services have stopped, or
-// they timeout after the timeout value given to the Supervisor at creation.
+// they timeout after the timeout value given to the Supervisor at
+// creation.
 func (s *Supervisor) Stop() {
-	done := make(chan struct{})
-	if s.sendControl(stopSupervisor{done}) {
-		<-done
-	}
+	s.StopWithReport()
 }
 
 func (s *Supervisor) handleFailedService(id serviceID, err interface{}, stacktrace []byte) {
@@ -589,10 +648,11 @@ func (s *Supervisor) runService(service Service, id serviceID) {
 	}()
 }
 
-func (s *Supervisor) removeService(id serviceID, notificationChan chan struct{}, removedChan chan supervisorMessage) {
+func (s *Supervisor) removeService(id serviceID, notificationChan chan struct{}) {
 	namedService, present := s.services[id]
 	if present {
 		delete(s.services, id)
+
 		s.servicesShuttingDown[id] = namedService
 		go func() {
 			successChan := make(chan struct{})
@@ -610,7 +670,7 @@ func (s *Supervisor) removeService(id serviceID, notificationChan chan struct{},
 			case <-s.getAfterChan(s.timeout):
 				s.LogBadStop(s, namedService.Service, namedService.name)
 			}
-			removedChan <- serviceTerminated{id}
+			s.notifyServiceDone <- id
 		}()
 	} else {
 		if notificationChan != nil {
@@ -619,8 +679,8 @@ func (s *Supervisor) removeService(id serviceID, notificationChan chan struct{},
 	}
 }
 
-func (s *Supervisor) stopSupervisor() {
-	notifyDone := make(chan serviceID)
+func (s *Supervisor) stopSupervisor() UnstoppedServiceReport {
+	notifyDone := make(chan serviceID, len(s.services))
 
 	for id := range s.services {
 		namedService, present := s.services[id]
@@ -640,6 +700,8 @@ SHUTTING_DOWN_SERVICES:
 		select {
 		case id := <-notifyDone:
 			delete(s.servicesShuttingDown, id)
+		case serviceID := <-s.notifyServiceDone:
+			delete(s.servicesShuttingDown, serviceID)
 		case <-timeout:
 			for _, namedService := range s.servicesShuttingDown {
 				s.LogBadStop(s, namedService.Service, namedService.name)
@@ -651,6 +713,20 @@ SHUTTING_DOWN_SERVICES:
 	}
 
 	close(s.liveness)
+
+	if len(s.servicesShuttingDown) == 0 {
+		return nil
+	} else {
+		report := UnstoppedServiceReport{}
+		for serviceID, serviceWithName := range s.servicesShuttingDown {
+			report = append(report, UnstoppedService{
+				Service:      serviceWithName.Service,
+				Name:         serviceWithName.name,
+				ServiceToken: ServiceToken{uint64(s.id)<<32 | uint64(serviceID)},
+			})
+		}
+		return report
+	}
 }
 
 // String implements the fmt.Stringer interface.
@@ -658,6 +734,11 @@ func (s *Supervisor) String() string {
 	return s.Name
 }
 
+// sendControl abstracts checking for the supervisor to still be running
+// when we send a message. This way, if someone does call Stop twice on a
+// supervisor or call stop in one goroutine while calling Stop in another,
+// the goroutines trying to call methods on a stopped supervisor won't hang
+// forever and leak.
 func (s *Supervisor) sendControl(sm supervisorMessage) bool {
 	select {
 	case s.control <- sm:
