@@ -214,17 +214,7 @@ func TestRunningAlreadyRunning(t *testing.T) {
 
 	// ensure the supervisor has made it to its main loop
 	s.sync()
-	var errored bool
-	func() {
-		defer func() {
-			if r := recover(); r != nil {
-				errored = true
-			}
-		}()
-
-		s.Serve()
-	}()
-	if !errored {
+	if !panics(s.Serve) {
 		t.Fatal("Supervisor failed to prevent itself from double-running.")
 	}
 }
@@ -392,7 +382,7 @@ func TestRemovingHungService(t *testing.T) {
 	<-service.started
 	service.take <- Hang
 
-	s.Remove(sToken)
+	_ = s.Remove(sToken)
 	resumeChan <- time.Time{}
 
 	<-failNotify
@@ -428,6 +418,33 @@ func TestRemoveService(t *testing.T) {
 	}
 }
 
+func TestServiceReport(t *testing.T) {
+	t.Parallel()
+
+	s := NewSimple("Top")
+	s.timeout = time.Millisecond
+	service := NewService("ServiceName")
+
+	id := s.Add(service)
+	go s.Serve()
+
+	<-service.started
+	service.take <- Hang
+
+	report := s.StopWithReport()
+	if !reflect.DeepEqual(report, UnstoppedServiceReport{
+		{service, "ServiceName", id},
+	}) {
+		t.Fatal("did not get expected stop service report")
+	}
+
+	// coverage testing; StopWithReport on a stopped supervisor returns a
+	// nil.
+	if s.StopWithReport() != nil {
+		t.Fatal("calling StopWithReport on a stopped supervisor doesn't work")
+	}
+}
+
 func TestFailureToConstruct(t *testing.T) {
 	t.Parallel()
 
@@ -611,6 +628,31 @@ func TestRemoveAndWait(t *testing.T) {
 	}
 }
 
+func TestStopSupervisorPanic(t *testing.T) {
+	t.Parallel()
+
+	s := NewSimple("test stop panic supervisor")
+	s.Stop()
+	if s.state != terminated {
+		t.Fatal("stopping server didn't go to the terminated state")
+	}
+	// this should return because it should come back having done nothing
+	s.Serve()
+}
+
+func TestSupervisorManagementIssue35(t *testing.T) {
+	s := NewSimple("issue 35")
+
+	for i := 1; i < 100; i++ {
+		s2 := NewSimple("test")
+		s.Add(s2)
+	}
+
+	s.ServeBackground()
+	// should not have any panics
+	s.Stop()
+}
+
 func TestCoverage(t *testing.T) {
 	New("testing coverage", Spec{
 		LogBadStop: func(*Supervisor, Service, string) {},
@@ -629,10 +671,45 @@ func TestCoverage(t *testing.T) {
 	})
 }
 
+func TestStopAfterRemoveAndWait(t *testing.T) {
+	t.Parallel()
+
+	var badStopError error
+
+	s := NewSimple("main")
+	s.timeout = time.Second
+	s.LogBadStop = func(sup *Supervisor, _ Service, name string) {
+		badStopError = fmt.Errorf("%s: Service %s failed to terminate in a timely manner", sup.Name, name)
+	}
+	s.ServeBackground()
+
+	service := NewService("A1")
+	token := s.Add(service)
+
+	<-service.started
+	service.take <- UseStopChan
+
+	err := s.RemoveAndWait(token, time.Second)
+	if err != nil {
+		t.Fatal("Happy case for RemoveAndWait failed: " + err.Error())
+	}
+	<-service.stop
+
+	s.Stop()
+
+	if badStopError != nil {
+		t.Fatal("Unexpected timeout while stopping supervisor: " + badStopError.Error())
+	}
+}
+
 // http://golangtutorials.blogspot.com/2011/10/gotest-unit-testing-and-benchmarking-go.html
 // claims test function are run in the same order as the source file...
 // I'm not sure if this is part of the contract, though. Especially in the
 // face of "t.Parallel()"...
+//
+// This is also why all the tests must go in this file; this test needs to
+// run last, and the only way I know to even hopefully guarantee that is to
+// have them all in one file.
 func TestEverMultistarted(t *testing.T) {
 	if everMultistarted {
 		t.Fatal("Seem to have multistarted a service at some point, bummer.")
